@@ -54,7 +54,7 @@ import warnings
 
 import os
 from errno import EALREADY, EINPROGRESS, EWOULDBLOCK, ECONNRESET, EINVAL, \
-     ENOTCONN, ESHUTDOWN, EISCONN, EBADF, ECONNABORTED, EPIPE, EAGAIN, \
+     ENOTCONN, ESHUTDOWN, EISCONN, EBADF, ECONNABORTED, EPIPE, EAGAIN, EINTR, \
      errorcode
 
 _DISCONNECTED = frozenset({ECONNRESET, ENOTCONN, ESHUTDOWN, ECONNABORTED, EPIPE,
@@ -127,7 +127,7 @@ def poll(timeout=0.0, map=None):
         map = socket_map
     if map:
         r = []; w = []; e = []
-        for fd, obj in list(map.items()):
+        for idx, (fd, obj) in enumerate (list(map.items())):
             is_r = obj.readable()
             is_w = obj.writable()
             if is_r:
@@ -137,6 +137,7 @@ def poll(timeout=0.0, map=None):
                 w.append(fd)
             if is_r or is_w:
                 e.append(fd)
+
         if [] == r == w == e:
             time.sleep(timeout)
             return
@@ -187,7 +188,64 @@ def poll2(timeout=0.0, map=None):
                 continue
             readwrite(obj, flags)
 
-poll3 = poll2                           # Alias for backward compatibility
+def epoll(timeout=0.0, map=None):
+    if map is None:
+        map = socket_map
+    pollster = select.epoll()
+    if map:
+        for fd, obj in list(map.items()):
+            flags = 0
+            if obj.readable():
+                flags |= select.POLLIN | select.POLLPRI
+            if obj.writable() and not obj.accepting:
+                flags |= select.POLLOUT
+            if flags:
+                # Only check for exceptions if object was either readable or writable
+                flags |= select.POLLERR | select.POLLHUP | select.POLLNVAL
+                pollster.register(fd, flags)
+
+        try:
+            r = pollster.poll(timeout)
+        except select.error as err:
+            if err.args[0] != EINTR:
+                raise
+            r = []
+
+        for fd, flags in r:
+            obj = map.get(fd)
+            if obj is None:
+                continue
+            readwrite(obj, flags)
+
+def kqueue(timeout=0.0, map=None):
+    if map is None:
+        map = socket_map
+    if map:
+        kqueue = select.kqueue()
+        flags = select.KQ_EV_ADD | select.KQ_EV_ENABLE
+        selectables = 0
+        for fd, obj in map.items():
+            filter = 0
+            if obj.readable():
+                filter |= select.KQ_FILTER_READ
+            if obj.writable():
+                filter |= select.KQ_FILTER_WRITE
+            if filter:
+                ev = select.kevent(fd, filter=filter, flags=flags)
+                kqueue.control([ev], 0)
+                selectables += 1
+
+        events = kqueue.control(None, selectables, timeout)
+        for event in events:
+            fd = event.ident
+            obj = map.get(fd)
+            if obj is None:
+                continue
+            if event.filter == select.KQ_FILTER_READ:
+                read(obj)
+            if event.filter == select.KQ_FILTER_WRITE:
+                write(obj)
+        kqueue.close()
 
 def loop(timeout=30.0, use_poll=False, map=None, count=None):
     if map is None:
